clear 
close all
load('traiX.mat');
load('queryX.mat');
load('T.mat');

%%% 正則化のパラメータ
reg.type = 't'; %'t':Tikhonov正則化，'r':ランク低減(TSVD)，'m':修正２次
reg.mu = 1e-6;  % 正則化パラメータ
reg.rank = 9;   % ランク
reg.delta = 1e-3; % 疑似ベイズのdelta

ntrai = size(trai.X,2); %標本数
d = size(trai.X,1); %次元数
nquery = length( query.label); %テスト標本数
nclass = max(trai.label);
imgsize = [112 92];

for c=1:nclass
  %% 学習用標本から各クラスの平均，分散共分散を求める
  index = find( trai.label == c );
  Class(c).m = mean( trai.X(:,index), 2 );
  Class(c).S = cov( trai.X(:,index)' ); 
  
  %% 識別
  switch reg.type
   case 't'
    A = inv(chol( Class(c).S + reg.mu * eye(d) )');
    b = sum( log( eig(Class(c).S + reg.mu * eye(d)) ) );
   case 'r'
    [V D] = eig(Class(c).S);
    [d index] = sort(diag(D),'descend');
    A = diag( 1./sqrt(d(1:reg.rank)) ) * V(:, index(1:reg.rank))';
    b = sum( log( d(1:reg.rank) ) );
   case 'm'
    [V D] = eig(Class(c).S);
    A = diag( 1./ sqrt( max( reg.delta, diag(D) ) ) ) * V';
    b = sum( log( max( reg.delta, diag(D) ) ) );
   otherwise
    error(sprintf('reg.type = %c is not implemented\n',reg.type));
  end
  
  %% 事後確率の対数を計算する
  feature(c,:) = -sum( ( A*(query.X - Class(c).m*ones(1,nquery)) ).^2 ) - b;
end

[tmp eclass] = max(feature);
accuracy = sum(eclass(:) == query.label(:))/nquery;
fprintf('accuracy = %.2f %%\n',accuracy*100 );

%% 誤識別したものを表示
index = find( eclass(:) ~= query.label(:) );
figure,colormap('gray');
for ii=1:min(length(index),9)
  subplot(3,3,ii);
  imagesc( reshape( T.m + T.U*query.X(:,index(ii)), imgsize(1), imgsize(2) ) );
end
