% makedata.m
% 次元削減後のテストパターンと訓練パターン
% およびそれらに対応するラベルをファイルに出力

clear all;
close all

cum_r=0.9;   % 累積寄与率 (0<cum_r<=1)
queryset=10; % queryset番目の画像をテスト画像にする (1<=queryset<=10)

nclass=40;   % クラス数
ntrai=360;   % 訓練画像数
nquery=40;   % テスト画像数
nj=10;       % クラスあたりの画像数
d=10304;     % 画像データの次元数

% 訓練データ用行列
TI=zeros(d,ntrai);          % 訓練画像を並べた行列
trai_label=zeros(ntrai,1);  % 訓練画像のラベル
% テストデータ用行列
QI=zeros(d,nquery);          % テスト画像を並べた行列
query_label=zeros(nquery,1); % テスト画像のラベル

% T,Qとラベルの作成
count=1;
for jj = 1 : nclass
	for ii = 1 : nj
	 	% 画像のファイル名の指定
		FILENAME=sprintf('./att_faces/s%d/%d.pgm',jj,ii);
		IMG=pgmread(FILENAME); 	% 画像の読み込み
		if ii == queryset % 各人のqueryset番目の画像をテスト画像に
			QI(:,jj)=IMG(:);
			query_label(jj)=jj;
		else       % それ以外は訓練画像に
			TI(:,count)=IMG(:);     % 画像をベクトル化してTIに保存
			trai_label(count)=jj;   % 画像に対応するラベルを保存
			count=count+1;
		end
	end
end
fprintf(2,'creating image matrices and labels -> finish \n');

% 訓練パターンから固有ベクトルの変換公式を利用して
% d次元の固有ベクトルを並べたUを算出

m=mean(TI,2);           % 平均ベクトル
X=TI-repmat(m,1,ntrai); % Tからmを引く
N = X'*X;               % n x nの小さい行列Nを計算
maxr = rank(N);         % Nの最大階数を算出

[eig_vec eig_val] = eig(N); % Nの固有値と固有ベクトルを求める
[value index]=sort(diag(eig_val),'descend'); % 固有値を降順に並べ替え

% 累積寄与率がcum_r以上となる次元数rを求める
for ii = 1 : maxr
	if sum(value(1:ii))./sum(value)>cum_r
		r=ii;
		break;
	end
end
U = X * eig_vec(:,index(1:r)) * diag(sqrt(1./value(1:r)) );

% 次元削減後のパターンと対応するラベルを構造体に格納
trai.X  = U' * X;                       % 次元削減後の訓練パターン
trai.label=trai_label;                  % 訓練パターンのラベル
query.X = U' * (QI-repmat(m,1,nquery)); % 次元削減後のテスト画像
query.label = query_label;              % テストパターンのラベル

fprintf(2,'dimension reduction -> finish\n');


% 平均mとUを構造体に格納
T.m=m;
T.U=U;

% 次元削減後の訓練パターンとラベルをファイルに保存
save('-mat','traiX.mat','trai');
% 次元削減後のテストパターンとラベルをファイルに保存
save('-mat','queryX.mat','query');
% mとUをファイルに保存
save('-mat','T.mat','T');

fprintf(2,'file output -> finish\n');

% figureの初期化
figure(1),clf,colormap(gray)
% mとUの一部を表示
image=reshape(m,112,92);
figure(1),subplot(1,4,1),imagesc(image); % mの表示
image=reshape(U(:,1),112,92);
figure(1),subplot(1,4,2),imagesc(image); % U(:,1)の表示
image=reshape(U(:,2),112,92);
figure(1),subplot(1,4,3),imagesc(image); % U(:,2)の表示
image=reshape(U(:,3),112,92);
figure(1),subplot(1,4,4),imagesc(image); % U(:,3)の表示

% 固有値の変化をプロット
figure(2),plot(value);
xlabel('Dimension');,ylabel('Eigenvalue');

% 最初の3人の原訓練パターンとUによる展開画像を表示
figure(3),clf,colormap(gray)
for ii=1:3
	IMG=TI(:,(ii-1).*9+1);
	subplot(3,2,ii.*2-1);
  imagesc( reshape(IMG, 112, 92) );
	IMG=m+U*trai.X(:,(ii-1).*9+1);
  subplot(3,2,ii.*2);
  imagesc( reshape(IMG, 112, 92) );
end

% 最初の3人の原テストパターンとUによる展開画像を表示
figure(4),clf,colormap(gray)
for ii=1:3
	IMG=QI(:,ii);
	subplot(3,2,ii.*2-1);
  imagesc( reshape(IMG, 112, 92) );
	IMG=m+U*query.X(:,ii);
  subplot(3,2,ii.*2);
  imagesc( reshape(IMG, 112, 92) );
end
